SystemOrganization addCategory: #'Xtreams-Terminals'!

XTWriteStream subclass: #XTBlockClosureWriteStream
	instanceVariableNames: 'contentsSpecies closeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTBlockClosureWriteStream commentStamp: '<historical>' prior: 0!
Write stream on a single argument block, the block is evaluated for each element written with the element assigned as the argument of the evaluation.
{{{
	"Transcript as an xtream"
	[ :x | Transcript nextPut: x ] writing write: 'Hello World!!'
}}}
{{{
	"/dev/null"
	[ :x | ] writing write: 'Hello World!!'
}}}

Instance Variables
	contentsSpecies	<Class> species for collections of elements of this stream
	closeBlock	<BlockClosure> invoked in response to the #close message, allows customizing the close behavior

!

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/13/2010 20:08'!
close
	closeBlock cull: self! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
closeBlock
	^closeBlock! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
closeBlock: aBlock
	closeBlock := aBlock! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
contentsSpecies
	^contentsSpecies! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
contentsSpecies: anObject
	contentsSpecies := anObject! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
flush! !

!XTBlockClosureWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:59'!
on: aBlockClosure
	super on: aBlockClosure.
	contentsSpecies := Array.
	closeBlock := []! !

!XTBlockClosureWriteStream methodsFor: 'accessing' stamp: 'MartinKobetic 1/11/2011 17:53'!
write: anInteger from: aSequenceableCollection at: startIndex
	| count |
	count := 0.
	[[count < anInteger] whileTrue:
		[destination value: (aSequenceableCollection at: startIndex + count).
		count := count + 1]]
			on: Incomplete do: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
	^anInteger! !

XTWriteStream subclass: #XTBufferWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTBufferWriteStream commentStamp: '<historical>' prior: 0!
Write stream on a Buffer. Usually used in tandem with a read stream to access buffer contents conveniently.
{{{
	buffer := ElasticBuffer on: String new.
	bufferIn := buffer writing.
	bufferOut := buffer reading.
	100000 timesRepeat: [ bufferIn write: 'Hello World'. bufferOut read: 11 ].
	buffer cacheSize  
}}}
!

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := destination writeSkip: anInteger.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := (destination writeSkip: anInteger negated) negated.
	count = anInteger ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:03'!
available
	^destination writeSize! !

!XTBufferWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:02'!
close
	destination close! !

!XTBufferWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:02'!
contentsSpecies
	^destination contentsSpecies! !

!XTBufferWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:02'!
flush! !

!XTBufferWriteStream methodsFor: 'accessing' stamp: 'nice 7/8/2011 23:54'!
insert: anInteger from: aSequenceableCollection at: startIndex
	^destination insert: anInteger from: aSequenceableCollection at: startIndex! !

!XTBufferWriteStream methodsFor: 'testing' stamp: 'nice 10/9/2010 22:03'!
isPositionable
	^true! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:03'!
length
	^destination activeSize! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:03'!
position
	^destination writePosition! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition
	| available |
	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	available := aPosition min: destination activeSize.
	destination writePosition: available.
	available = aPosition ifTrue: [^aPosition ].
	(Incomplete count: available) raise! !

!XTBufferWriteStream methodsFor: 'accessing' stamp: 'nice 10/9/2010 22:02'!
put: anObject
	^destination put: anObject! !

!XTBufferWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
skip: anInteger
	| count |
	count := destination writeSkip: anInteger.
	count ~= anInteger ifTrue: [(Incomplete count: count) raise]! !

!XTBufferWriteStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:18'!
write: anInteger from: aSequenceableCollection at: startIndex
	^destination write: anInteger from: aSequenceableCollection at: startIndex! !

XTWriteStream subclass: #XTCollectionWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTCollectionWriteStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 00:38'!
close! !

!XTCollectionWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 00:39'!
put: anObject
	destination add: anObject! !

!XTCollectionWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 00:59'!
write: anInteger from: aSequenceableCollection at: startIndex
	startIndex to: startIndex + anInteger - 1 do: [:index |
		destination add: (aSequenceableCollection at: index)].
	^anInteger! !

XTWriteStream subclass: #XTExternalWriteStream
	instanceVariableNames: 'contentsSpecies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTExternalWriteStream commentStamp: '<historical>' prior: 0!
Used to write to BlockableIOAccessors (e.g sockets or pipes). This is a binary stream (consumes bytes/ByteArrays).

{{{
	[ :in :out |
		[	out writing write: 'Hello'; close.
			in reading read: 5
		] ensure: [ in close. out close ]
	] valueWithArguments: SocketAccessor openPair
}}}
{{{
	[ :in :out |
		[	out writing write: 'Hello'; close.
			in reading read: 5
		] ensure: [ in close. out close ]
	] valueWithArguments: OSSystemSupport concreteClass pipeAccessorClass openPair
}}}

Instance Variables
	contentsSpecies	<Class> species for collections of elements of this stream

!

!XTExternalWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:20'!
close
	destination closed ifTrue: [^self].
	destination close! !

!XTExternalWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:20'!
contentsSpecies
	^contentsSpecies! !

!XTExternalWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:20'!
contentsSpecies: aClass
	contentsSpecies := aClass! !

!XTExternalWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:20'!
flush
	destination closed ifTrue: [^self].
	destination commit! !

!XTExternalWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:20'!
on: anAccessor
	super on: anAccessor.
	contentsSpecies := ByteArray! !

!XTExternalWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:00'!
write: anInteger from: aSequenceableCollection at: startIndex
	| count wrote |
	anInteger isZero ifTrue: [^0].
	count := 0.
	[count < anInteger] whileTrue:
		[wrote := destination next: anInteger - count putAll: aSequenceableCollection startingAt: startIndex + count.
		wrote isZero ifTrue: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
		count := count + wrote].
	^anInteger! !

XTExternalWriteStream subclass: #XTFileWriteStream
	instanceVariableNames: 'position isPositionable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTFileWriteStream commentStamp: '<historical>' prior: 0!
Writes to a file. File write streams can be created via the usual #writing message or via #appending which opens the file in appending mode. In appending mode, you cannot position the stream before the end of the file contents, so you can never overwrite existing contents. In writing mode, the file will be truncated at stream's current position when #close is called. To keep the entire contents of the file, use -= 0 to skip to the end before closing. This behavior is different from the classic streams which would erase the contents of the file on opening. The stream is binary and naturally positionable.
{{{
	| file |
	file := '/dev/shm/xtreams-test' asFilename.
	[	file writing write: 'Hello'; close.
		file appending write: ' World!!'; close.
		file contentsOfEntireFile.
	] ensure: [ file delete ]
}}}
It is also possible to send #reading or #writing to a pre-opened IOAcccessor if some other opening mode configuration is desirable. For example to emulate the classic write stream opening behavior, you can use the following:
{{{
	(IOAccessor openFileNamed: '/dev/shm/xtreams-test' 
		direction: IOAccessor writeOnly
		creation: IOAccessor truncateOrCreate
	) writing close
}}}

Instance Variables
	position	<Integer> current position of the stream
	isPositionable	<Boolean> indicates that the file is open in append mode
	contentsSpecies	<Class> species for collections of elements of this stream

!

!XTFileWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	self isPositionable ifFalse: [self error: 'This stream is not positionable.'].
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := self available min: anInteger.
	position := position + count.
	destination seekTo: position.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTFileWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	self isPositionable ifFalse: [self error: 'This stream is not positionable.'].
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := position min: anInteger.
	position := position - count.
	destination seekTo: position.
	anInteger = count ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTFileWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/15/2010 20:39'!
close
	destination closed ifTrue: [^self].
	destination truncateTo: position.
	destination close! !

!XTFileWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:00'!
insert: anInteger from: aSequenceableCollection at: startIndex
	^self shouldNotImplement.

	"While it might seem desireable to be able to insert in to a file, the reality is you can only do so if you can read and write to the file. if you can read and write to the file, the fileSize becomes indeterminate. The only reasonable way to insert in to a file is to memory map it and access it with a PointerWriteStream instead."! !

!XTFileWriteStream methodsFor: 'testing' stamp: 'nice 10/12/2010 02:39'!
isPositionable
	^isPositionable! !

!XTFileWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:38'!
isPositionable: aBoolean
	"Only set to false when the file is open in append mode."

	isPositionable := aBoolean! !

!XTFileWriteStream methodsFor: 'seeking' stamp: 'nice 10/12/2010 02:39'!
length
	self isPositionable ifFalse: [self error: 'This stream is not positionable.'].
	^destination fileSize! !

!XTFileWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/14/2010 21:52'!
on: anAccessor
	super on: anAccessor.
	isPositionable := true.
	position := 0! !

!XTFileWriteStream methodsFor: 'seeking' stamp: 'nice 10/12/2010 02:39'!
position
	self isPositionable ifFalse: [self error: 'This stream is not positionable.'].
	^position! !

!XTFileWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition
	| available |
	self isPositionable ifFalse: [self error: 'This stream is not positionable.'].
	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	available := aPosition min: self length.
	destination seekTo: available.
	position := available.
	available = aPosition ifTrue: [ ^aPosition ].
	(Incomplete count: available) raise! !

!XTFileWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:00'!
write: anInteger from: aSequenceableCollection at: startIndex
	| count wrote |
	anInteger isZero ifTrue: [^0].
	count := 0.
	[count < anInteger] whileTrue:
		[wrote := destination writeFrom: aSequenceableCollection startingAt: startIndex + count count: anInteger.
		wrote isZero ifTrue: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
		count := count + wrote.
		position := position + wrote].
	^anInteger! !

XTWriteStream subclass: #XTNullWriteStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTNullWriteStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 00:54'!
close! !

!XTNullWriteStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 00:45'!
contentsSpecies
	^Array! !

!XTNullWriteStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 00:54'!
flush! !

!XTNullWriteStream methodsFor: 'private' stamp: 'nice 7/9/2011 00:46'!
streamingInsert: anInteger from: aReadStream
	aReadStream ++ anInteger! !

!XTNullWriteStream methodsFor: 'private' stamp: 'nice 7/9/2011 00:47'!
streamingInsertFrom: aReadStream
	aReadStream -= 0! !

!XTNullWriteStream methodsFor: 'private' stamp: 'nice 7/9/2011 00:47'!
streamingWrite: anInteger from: aReadStream
	aReadStream ++ anInteger! !

!XTNullWriteStream methodsFor: 'private' stamp: 'nice 7/9/2011 00:47'!
streamingWriteFrom: aReadStream
	aReadStream -= 0! !

!XTNullWriteStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:01'!
write: anInteger from: aSequenceableCollection at: startIndex
	^anInteger! !

XTWriteStream subclass: #XTSequenceableCollectionWriteStream
	instanceVariableNames: 'position length'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTSequenceableCollectionWriteStream commentStamp: '<historical>' prior: 0!
Write stream on a sequenceable collection. The collection is grown automatically to accommodate any elements written. Closing a collection write stream will truncate the collection to the current stream position. This behavior is useful as a replacement for the traditional #contents message. The contents can be accessed with the #terminal message after the stream is closed.
{{{
	String new writing write: 'Hello World'; -- 6; close; terminal
}}}

Instance Variables
	position	<Integer> current position of the stream
	length	<Integer> number of valid elements in the destination

!

!XTSequenceableCollectionWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := self available min: anInteger.
	position := position + count.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTSequenceableCollectionWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := position min: anInteger.
	position := position - count.
	count = anInteger ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTSequenceableCollectionWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/16/2010 09:44'!
close
	position < destination size ifTrue: [destination := destination copyFrom: 1 to: position]! !

!XTSequenceableCollectionWriteStream methodsFor: 'accessing' stamp: 'nice 10/9/2010 22:07'!
contents
	^destination copyFrom: 1 to: position! !

!XTSequenceableCollectionWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:14'!
contentsSpecies
	^destination species! !

!XTSequenceableCollectionWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:14'!
flush! !

!XTSequenceableCollectionWriteStream methodsFor: 'accessing' stamp: 'nice 7/8/2011 23:54'!
insert: anInteger from: aSequenceableCollection at: startIndex
	"Implementation note: this code does a memory move to the right.
	Since Squeak primitive #replaceFrom:to:with: is implemented with a memcpy rather than a memmove, it cannot work.
	We have to buffer the copy in a motionBuffer temporary."
	| motionBuffer |
	motionBuffer := destination.
	destination := destination grownBy: (anInteger max: destination size).
	destination replaceFrom: position + anInteger + 1 to: length + anInteger with: motionBuffer startingAt: position + 1.
	destination replaceFrom: position + 1 to: position + anInteger with: aSequenceableCollection startingAt: startIndex.
	position := position + anInteger.
	length := length + anInteger.
	^anInteger! !

!XTSequenceableCollectionWriteStream methodsFor: 'testing' stamp: 'nice 10/9/2010 22:17'!
isPositionable
	^true! !

!XTSequenceableCollectionWriteStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:17'!
length
	^length! !

!XTSequenceableCollectionWriteStream methodsFor: 'initialize-release' stamp: 'MartinKobetic 12/30/2010 23:25'!
on: aSequenceableCollection
	super on: aSequenceableCollection.
	position := 0.
	length := 0! !

!XTSequenceableCollectionWriteStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:17'!
position
	^position! !

!XTSequenceableCollectionWriteStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition
	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	position := aPosition min: self length.
	position = aPosition ifTrue: [^aPosition].
	(Incomplete count: position) raise! !

!XTSequenceableCollectionWriteStream methodsFor: 'accessing' stamp: 'nice 10/16/2010 09:51'!
put: anObject
	| free |
	free := destination size - position.
	free < 1 ifTrue: [destination := destination grownBy: (destination size max: 1)].
	position := position + 1.
	destination at: position put: anObject.
	length := length + 1.
	^anObject! !

!XTSequenceableCollectionWriteStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingInsert: anInteger from: aStreamable
	"Implementation note: this code does a memory move to the right.
	Since Squeak primitive #replaceFrom:to:with: is implemented with a memcpy rather than a memmove, it cannot work.
	We have to buffer the copy in motionBuffer temporary."
	| reading motionBuffer |
	reading := aStreamable reading.
	motionBuffer := destination.
	destination := destination grownBy: (destination size max: anInteger).
	destination replaceFrom: position + anInteger to: anInteger + motionBuffer size with: motionBuffer startingAt: position.
	[reading read: anInteger into: destination at: position + 1]
		on: Incomplete do: [:error | position := position + error count. error pass].
	position := position + anInteger! !

!XTSequenceableCollectionWriteStream methodsFor: 'private' stamp: 'nice 10/9/2010 22:15'!
streamingInsertFrom: aStreamable
	self error: 'Not Yet Implemented'! !

!XTSequenceableCollectionWriteStream methodsFor: 'printing' stamp: 'nice 10/9/2010 22:14'!
streamingPrintOn: aStream
	| tail head |
	super streamingPrintOn: aStream.
	tail := position min: 5.
	head := (destination size - position) min: (10 - tail).
	aStream
		write: ' position: ';
		print: position;
		write: ' size: ';
		print: destination size;
		cr; tab;
		write: (position > 5 ifTrue: ['...'] ifFalse: ['']);
		print: (destination copyFrom: position + 1 - tail to: position);
		write: '  |  ';
		print: (destination copyFrom: position + 1 to: position + head);
		write: (destination size - position > (10 - tail) ifTrue: ['...'] ifFalse: [''])! !

!XTSequenceableCollectionWriteStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingWrite: anInteger from: aReadStream
	| available |
	available := destination size - position.
	available < anInteger ifTrue: [destination := destination grownBy: (destination size max: anInteger - available)].
	[aReadStream read: anInteger into: destination at: position + 1]
		on: Incomplete do: [:error |
			position := position + error count.
			position > length ifTrue: [length := position].
			error pass].
	position := position + anInteger.
	position > length ifTrue: [length := position].
	^anInteger! !

!XTSequenceableCollectionWriteStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingWriteFrom: aReadStream
	| available count chunkSize |
	count := 0.
	chunkSize := DefaultBufferSize.
	^[[available := destination size - position.
	available < chunkSize ifTrue: [destination := destination grownBy: (destination size max: chunkSize)].
	aReadStream read: chunkSize into: destination at: position + 1.
	position := position + chunkSize.
	count := count + chunkSize] repeat]
		on: Incomplete do: [:exception |
			position := position + exception count.
			position > length ifTrue: [length := position].
			count + exception count]! !

!XTSequenceableCollectionWriteStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:28'!
write: anInteger from: aSequenceableCollection at: startIndex
	| available |
	available := destination size - position.
	available < anInteger ifTrue: [destination := destination grownBy: (destination size max: anInteger - available)].
	destination replaceFrom: position + 1 to: position + anInteger with: aSequenceableCollection startingAt: startIndex.
	position := position + anInteger.
	position > length ifTrue: [length := position].
	^anInteger! !

XTWriteStream subclass: #XTSharedQueueWriteStream
	instanceVariableNames: 'contentsSpecies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTSharedQueueWriteStream commentStamp: '<historical>' prior: 0!
Write stream on a SharedQueue. Primarily used for data transfer between processes.
{{{
	queue := SharedQueue new.
	in := queue reading.
	out := queue writing.
	received := Array new writing.
	done := Semaphore new.
	consumer :=	
		[ | size |
			[	(size := in get) isZero
			] whileFalse: [ | word |
				word := ByteString new: size.
				in read: size into: word.
				received put: word ].
			done signal.
		] fork.
	#(one two three four) do: [ :word | out put: word size; write: word ].
	out put: 0.
	done wait.
	received conclusion
}}}

Instance Variables
	contentsSpecies	<Class> species for collections of elements of this stream

!

!XTSharedQueueWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:25'!
close! !

!XTSharedQueueWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:25'!
contentsSpecies
	^contentsSpecies! !

!XTSharedQueueWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:25'!
contentsSpecies: aClass
	contentsSpecies := aClass! !

!XTSharedQueueWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:26'!
flush! !

!XTSharedQueueWriteStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:26'!
on: aDestination
	super on: aDestination.
	contentsSpecies := Array! !

!XTSharedQueueWriteStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:42'!
write: anInteger from: aSequenceableCollection at: startIndex
	startIndex to: startIndex + anInteger - 1 do: [:index | destination nextPut: (aSequenceableCollection at: index)].
	^anInteger! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'nice 10/14/2010 22:02'!
accepting
	"Return a read stream that produces new sockets from incoming connections."

	"^	ReadStream"

	"Listen for connections and close those connections immediately.
	| socket |
	socket := SocketAccessor newTCP.
	socket listenFor: 1.
	[socket acceping do: [:client | client close]]
		ensure: [socket close]
	"

	^[self accept] reading
		closeBlock: [self close];
		yourself! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'nice 10/14/2010 22:25'!
closed
	^self isUnconnectedOrInvalid! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'nice 10/14/2010 22:36'!
commit
	self isOtherEndClosed
		 ifFalse: [self sendDone
			ifFalse: ["What to do next ?" self waitForSendDoneFor: 10]]! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'MartinKobetic 1/14/2011 00:54'!
next: count putAll: buffer startingAt: startIndex
	"Immediately send data (no wait).
	Answer number of bytes written."
	self isConnected ifFalse: [^0].
	^self sendSomeData: buffer startIndex: startIndex count: count! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'MartinKobetic 1/13/2011 17:43'!
readInto: buffer
	"Only read available data - no wait.
	Answer number of bytes read."
	^self receiveDataInto: buffer startingAt: 1! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'nice 10/14/2010 22:01'!
reading
	^XTExternalReadStream on: self! !

!Socket methodsFor: '*Xtreams-Terminals' stamp: 'nice 10/14/2010 22:01'!
writing
	^XTExternalWriteStream on: self! !

!FileDirectory methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 20:41'!
appending
	| handle |
	handle := XTIOHandle forFiles open: pathName asSqueakPathName forWrite: true.
	^handle writing
		position: handle fileSize;
		isPositionable: false;
		yourself! !

!FileDirectory methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 21:06'!
reading
	"we should check for a directory first..."
	
	^(XTIOHandle forFiles new open: pathName asSqueakPathName forWrite: false) reading! !

!FileDirectory methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 20:41'!
writing
	^(XTIOHandle forFiles new open: pathName asSqueakPathName forWrite: true) writing! !

!ContextPart methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 00:51'!
reading
	| context |
	(context := self class new) swapSender: self.
	^[context == nil ifTrue: [Incomplete raise].
	   context := context sender]
		reading! !

!Random methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/9/2010 22:20'!
reading
	^[self next] reading
		contentsSpecies: Array;
		yourself! !

!SharedQueue methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/9/2010 23:19'!
reading
	^XTSharedQueueReadStream on: self! !

!SharedQueue methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/9/2010 23:19'!
writing
	^XTSharedQueueWriteStream on: self! !

!BlockClosure methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 01:07'!
reading
	self numArgs = 0 ifTrue: [^XTBlockClosureReadStream on: self].
	self numArgs = 1 ifTrue: [^XTBlockClosureGenerateStream on: self].
	self error: 'More arguments than we can poke a stick at!!'! !

!BlockClosure methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 01:08'!
writing
	self numArgs = 1 ifTrue: [^XTBlockClosureWriteStream on: self].
	self error: 'Must be a single argument block.'! !

!Collection methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 00:41'!
writing
	^XTCollectionWriteStream on: self! !

!XTBuffer methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/9/2010 21:56'!
reading
	^XTBufferReadStream on: self! !

!XTBuffer methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/9/2010 21:56'!
writing
	^XTBufferWriteStream on: self! !

!SequenceableCollection methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 00:34'!
reading
	^XTSequenceableCollectionReadStream on: self! !

!SequenceableCollection methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 00:35'!
writing
	^XTSequenceableCollectionWriteStream on: self! !

!FileUrl methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 21:01'!
appending
	^(FileDirectory default / self pathForFile) appending! !

!FileUrl methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 21:01'!
reading
	^(FileDirectory default / self pathForFile) reading! !

!FileUrl methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 10/12/2010 21:01'!
writing
	^(FileDirectory default / self pathForFile) writing! !

XTReadStream subclass: #XTBlockClosureGenerateStream
	instanceVariableNames: 'process current contentsSpecies writing readingSemaphore writingSemaphore closed'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTBlockClosureGenerateStream commentStamp: 'nice 7/9/2011 01:04' prior: 0!
Read stream on a one argument block, evaluates the block once for the life time of the stream. Once the block finishes execution, the stream is closed. The block is expected to write to the block argument, which will block exectuion until elements are read from the stream.
{{{
	"In this example, we have a -hard- loop, not using collection protocol, which will only run one element at a time."
	[:out | 1 to: 10 do: [:i | out put: i]] reading read: 5.
}}}
{{{
	"Fibonacci"
	[:out | | a b x |
	a := 0. b := 1.
	[out put: a.
	x := a.
	a := b.
	b := b + x] repeat] reading ++ 500; get.
}}}

Instance Variables
	current	<Object>	the current element in the stream
	closed	<Boolean>	true if the block has finished execution
	contentsSpecies	<Class>	species for collections of elements of this stream
	process	<Process>	the process executing the block
	readingSemaphore	<Semaphore>	attempts to read will wait on this
	writingSemaphore	<Semaphore>	attempts to write will wait on this, signaled when an attempt to read is performed

!

!XTBlockClosureGenerateStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 01:04'!
close
	closed := true.
	process terminate.
	readingSemaphore signal! !

!XTBlockClosureGenerateStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 01:04'!
contentsSpecies
	^contentsSpecies! !

!XTBlockClosureGenerateStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 01:05'!
contentsSpecies: aClass
	contentsSpecies := aClass! !

!XTBlockClosureGenerateStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:05'!
get
	writingSemaphore signal.
	closed ifTrue: [Incomplete zero raise].
	readingSemaphore wait.
	^current! !

!XTBlockClosureGenerateStream methodsFor: 'initialize-release' stamp: 'nice 7/9/2011 01:05'!
on: aBlockClosure
	super on: aBlockClosure.
	contentsSpecies := Array.
	readingSemaphore := Semaphore new.
	writingSemaphore := Semaphore new.
	closed := false.
	process :=
		[aBlockClosure value: [:in |
			writingSemaphore wait.
			current := in.
			readingSemaphore signal] writing.
		closed := true] fork! !

!XTBlockClosureGenerateStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 01:06'!
read: anInteger into: aSequenceableCollection at: startIndex
	| count |
	count := 0.
	[[count < anInteger] whileTrue:
		[aSequenceableCollection at: startIndex + count put: self get.
		count := count + 1]]
			on: Incomplete do: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
	^anInteger! !

XTReadStream subclass: #XTBlockClosureReadStream
	instanceVariableNames: 'contentsSpecies closeBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTBlockClosureReadStream commentStamp: 'nice 7/9/2011 01:07' prior: 0!
Read stream on a zero argument block, evaluates the block once for each requested element, the result of the block evaluation is the element.
{{{
	"inifinite stream of ones"
	[ 1 ] reading read: 20
}}}
{{{
	"Fibonacci"
	| a b | a := 0. b := 1.
	[ | x | x := a. a := b. b := x + a. x ] reading ++ 500; get
}}}
{{{
	"Streaming over ObjectMemory"
	x := ObjectMemory someObject.
	[ x := ObjectMemory nextObjectAfter: x ] reading read: 5
}}}

Instance Variables
	contentsSpecies	<Class> species for collections of elements of this stream
	closeBlock	<BlockClosure> invoked in response to the #close message, allows customizing the close behavior

!

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/13/2010 20:08'!
close
	closeBlock cull: self! !

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:58'!
closeBlock
	^closeBlock! !

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:58'!
closeBlock: aBlock
	closeBlock := aBlock! !

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:58'!
contentsSpecies
	^contentsSpecies! !

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:58'!
contentsSpecies: anObject
	contentsSpecies := anObject! !

!XTBlockClosureReadStream methodsFor: 'accessing' stamp: 'nice 10/9/2010 21:57'!
get
	^source value! !

!XTBlockClosureReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 21:58'!
on: aBlockClosure
	super on: aBlockClosure.
	contentsSpecies := Array.
	closeBlock := []! !

!XTBlockClosureReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 1/11/2011 17:53'!
read: anInteger into: aSequenceableCollection at: startIndex
	| count |
	count := 0.
	[[count < anInteger] whileTrue:
		[aSequenceableCollection at: startIndex + count put: source value.
		count := count + 1]]
			on: Incomplete do: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
	^anInteger! !

XTReadStream subclass: #XTBufferReadStream
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTBufferReadStream commentStamp: '<historical>' prior: 0!
Read stream on a Buffer. Usually used in tandem with a write stream to access buffer contents conveniently.
{{{
	buffer := ElasticBuffer on: String new.
	bufferIn := buffer writing.
	bufferOut := buffer reading.
	100000 timesRepeat: [ bufferIn write: 'Hello World'. bufferOut read: 11 ].
	buffer cacheSize  
}}}
!

!XTBufferReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := source readSkip: anInteger.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTBufferReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := (source readSkip: anInteger negated) negated.
	count = anInteger ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTBufferReadStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:01'!
available
	^source readSize! !

!XTBufferReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:00'!
close! !

!XTBufferReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:00'!
contentsSpecies
	^source contentsSpecies! !

!XTBufferReadStream methodsFor: 'accessing' stamp: 'nice 10/9/2010 22:00'!
get
	^source get! !

!XTBufferReadStream methodsFor: 'testing' stamp: 'nice 10/9/2010 22:01'!
isPositionable
	^true! !

!XTBufferReadStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:01'!
length
	^source activeSize! !

!XTBufferReadStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:01'!
position
	^source readPosition! !

!XTBufferReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition
	| available |
	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	available := aPosition min: source activeSize.
	source readPosition: available.
	available = aPosition ifTrue: [^aPosition ].
	(Incomplete count: available) raise! !

!XTBufferReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:15'!
read: anInteger into: aSequenceableCollection at: startIndex
	source read: anInteger into: aSequenceableCollection at: startIndex.
	^anInteger! !

XTReadStream subclass: #XTExternalReadStream
	instanceVariableNames: 'cache cachePosition cacheDataSize'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTExternalReadStream commentStamp: '<historical>' prior: 0!
Used to read from BlockableIOAccessors (e.g sockets or pipes). Elements are read-ahead but only what's available. This is a binary stream (produces bytes/ByteArrays).

{{{
	[ :in :out |
		[	out writing write: 'Hello'; close.
			in reading read: 5
		] ensure: [ in close. out close ]
	] valueWithArguments: SocketAccessor openPair
}}}
{{{
	[ :in :out |
		[	out writing write: 'Hello'; close.
			in reading read: 5
		] ensure: [ in close. out close ]
	] valueWithArguments: OSSystemSupport concreteClass pipeAccessorClass openPair
}}}

Instance Variables
	cache	<ByteArray | ByteString> read-ahead buffer
	cachePosition	<SmallInteger> position in the buffer
	cacheDataSize	<SmallInteger> size of valid data in the buffer

!

!XTExternalReadStream methodsFor: 'initialize-release' stamp: 'MartinKobetic 12/30/2010 23:28'!
close
	super close.
	cache recycle! !

!XTExternalReadStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:23'!
contentsSpecies
	^cache class! !

!XTExternalReadStream methodsFor: 'initialize-release' stamp: 'MartinKobetic 1/1/2011 16:24'!
contentsSpecies: aClass
	cache ifNotNil: [ cache recycle ].
	cache := aClass newRecycledDefaultSize.
	cachePosition := 0.
	cacheDataSize := 0.! !

!XTExternalReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 1/11/2011 17:53'!
get
	cachePosition < cacheDataSize ifFalse: [
		cachePosition := 0.
		cacheDataSize := source readInto: cache.
		cacheDataSize isZero ifTrue: [ self incompleteRead: Incomplete zero ] ].
	^cache at: (cachePosition := cachePosition + 1)! !

!XTExternalReadStream methodsFor: 'private' stamp: 'nice 10/12/2010 02:24'!
incompleteRead: incomplete

	incomplete raise! !

!XTExternalReadStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:23'!
on: anAccessor
	super on: anAccessor.
	self contentsSpecies: ByteArray! !

!XTExternalReadStream methodsFor: 'accessing' stamp: 'nice 7/9/2011 00:55'!
read: anInteger into: aSequenceableCollection at: startIndex

	| index toRead |
	anInteger isZero ifTrue: [^0].
	index := startIndex.
	toRead := anInteger.
	[toRead > 0] whileTrue: [ | amount |
		cachePosition < cacheDataSize ifFalse: [
			cachePosition := 0.
			cacheDataSize := source readInto: cache.
			cacheDataSize isZero ifTrue: [ self incompleteRead: (Incomplete on: aSequenceableCollection count: anInteger - toRead at: startIndex) ] ].
		amount := (cacheDataSize - cachePosition) min: toRead.
		aSequenceableCollection replaceFrom: index to: index + amount - 1 with: cache startingAt: cachePosition + 1.
		cachePosition := cachePosition + amount.
		index := index + amount.
		toRead := toRead - amount ].
	^anInteger! !

XTExternalReadStream subclass: #XTFileReadStream
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTFileReadStream commentStamp: '<historical>' prior: 0!
Reads from a file. The stream is usually created by sending #reading to a Filename. However the actual terminal is an IOAccessor. The original filename is accessible through the IOAccessor. The stream is binary and is naturally positionable.
{{{
	| file |
	file := ObjectMemory imageName asFilename reading.
	[ file read: 13 ] ensure: [ file close ]
}}}
As a convenience, sending #reading to a Filename of a directory will create a stream of all filenames in that directory.
{{{
	'/tmp' asFilename reading rest
}}}

Instance Variables
	position	<Integer> current position of the stream

!

!XTFileReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := self available min: anInteger.
	position := position + count.
	source seekTo: position.
	cacheDataSize := cachePosition := 0.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTFileReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := position min: anInteger.
	position := position - count.
	source seekTo: position.
	cacheDataSize := cachePosition := 0.
	anInteger = count ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTFileReadStream methodsFor: 'accessing' stamp: 'nice 10/12/2010 02:27'!
get

	| object |
	object := super get.
	position := position + 1.
	^object! !

!XTFileReadStream methodsFor: 'private' stamp: 'nice 10/12/2010 02:28'!
incompleteRead: incomplete

	position := position + incomplete count.
	super incompleteRead: incomplete.! !

!XTFileReadStream methodsFor: 'testing' stamp: 'nice 10/12/2010 02:34'!
isPositionable
	^true! !

!XTFileReadStream methodsFor: 'seeking' stamp: 'nice 10/14/2010 23:04'!
length
	^source fileSize! !

!XTFileReadStream methodsFor: 'initialize-release' stamp: 'nice 10/12/2010 02:28'!
on: anAccessor
	super on: anAccessor.
	position := 0! !

!XTFileReadStream methodsFor: 'seeking' stamp: 'nice 10/12/2010 02:34'!
position
	^position! !

!XTFileReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition

	| available |
	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	available := aPosition min: self length.
	source seekTo: available.
	position := available.
	cacheDataSize := cachePosition := 0.
	available = aPosition ifTrue: [^aPosition].
	(Incomplete count: available) raise! !

!XTFileReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:35'!
read: anInteger into: aSequenceableCollection at: startIndex

	super read: anInteger into: aSequenceableCollection at: startIndex.
	position := position + anInteger.
	^anInteger! !

XTReadStream subclass: #XTSequenceableCollectionReadStream
	instanceVariableNames: 'position'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTSequenceableCollectionReadStream commentStamp: '<historical>' prior: 0!
Read stream on a sequenceable collection.
{{{
	(1 to: 10000) reading ++ 1000; read: 5
}}}

Instance Variables
	position	<Integer> current position of the stream

!

!XTSequenceableCollectionReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
++ anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self -- anInteger negated ].
	count := self available min: anInteger.
	position := position + count.
	count < anInteger ifTrue: [(Incomplete count: count) raise].
	^anInteger! !

!XTSequenceableCollectionReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
-- anInteger
	| count |
	anInteger < 0 ifTrue: [ ^self ++ anInteger negated ].
	count := position min: anInteger.
	position := position - count.
	count = anInteger ifTrue: [^anInteger].
	(Incomplete count: count) raise! !

!XTSequenceableCollectionReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:04'!
close! !

!XTSequenceableCollectionReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:05'!
contentsSpecies
	^source species! !

!XTSequenceableCollectionReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 1/11/2011 17:53'!
get
	position = source size ifTrue: [Incomplete zero raise].
	position := position + 1.
	^source at: position! !

!XTSequenceableCollectionReadStream methodsFor: 'testing' stamp: 'nice 10/9/2010 22:07'!
isPositionable
	^true! !

!XTSequenceableCollectionReadStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:06'!
length
	^source size! !

!XTSequenceableCollectionReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:05'!
on: aSequenceableCollection
	super on: aSequenceableCollection.
	position := 0! !

!XTSequenceableCollectionReadStream methodsFor: 'seeking' stamp: 'nice 10/9/2010 22:06'!
position
	" Return a bookmark for the current stream state. "

	^position! !

!XTSequenceableCollectionReadStream methodsFor: 'seeking' stamp: 'MartinKobetic 1/11/2011 17:53'!
position: aPosition

	aPosition < 0 ifTrue: [ Incomplete zero raise ].
	position := aPosition min: self length.
	position = aPosition ifTrue: [^aPosition ].
	(Incomplete count: position) raise! !

!XTSequenceableCollectionReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 1/11/2011 17:53'!
read: anInteger into: aSequenceableCollection at: startIndex
	| count |
	count := anInteger min: self available.
	aSequenceableCollection replaceFrom: startIndex to: startIndex + count - 1 with: source startingAt: position + 1.
	position := position + count.
	count < anInteger ifTrue: [(Incomplete on: aSequenceableCollection count: count at: startIndex) raise].
	^anInteger! !

!XTSequenceableCollectionReadStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingInsert: anInteger into: aWriteStream
	| count |
	anInteger isZero ifTrue: [^self].
	count := anInteger min: self available.
	aWriteStream insert: count from: source at: position + 1.
	position := position + count.
	count >= anInteger ifTrue: [^self].
	(Incomplete on: source count: count at: position - count + 1) raise! !

!XTSequenceableCollectionReadStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingInsertInto: aWriteStream
	| count |
	(count := self available) isZero ifTrue: [Incomplete zero raise].
	self streamingInsert: count into: aWriteStream.
	^count! !

!XTSequenceableCollectionReadStream methodsFor: 'printing' stamp: 'nice 10/9/2010 22:05'!
streamingPrintOn: aStream
	| tail head |
	super streamingPrintOn: aStream.
	tail := position min: 5.
	head := (source size - position) min: (10 - tail).
	aStream
		write: ' position: ';
		print: position;
		write: ' size: ';
		print: source size;
		cr; tab;
		write: (position > 5 ifTrue: ['...'] ifFalse: ['']);
		print: (source copyFrom: position + 1 - tail to: position);
		write: '  |  ';
		print: (source copyFrom: position + 1 to: position + head);
		write: (source size - position > (10 - tail) ifTrue: ['...'] ifFalse: [''])! !

!XTSequenceableCollectionReadStream methodsFor: 'private' stamp: 'MartinKobetic 1/11/2011 17:53'!
streamingWrite: anInteger into: aWriteStream
	| count |
	anInteger isZero ifTrue: [^0].
	count := anInteger min: self available.
	aWriteStream write: count from: source at: position + 1.
	position := position + count.
	count = anInteger ifTrue: [^anInteger].
	(Incomplete on: source count: count at: position - count + 1) raise! !

!XTSequenceableCollectionReadStream methodsFor: 'private' stamp: 'nice 10/9/2010 22:06'!
streamingWriteInto: aWriteStream
	| count |
	(count := self available) isZero ifTrue: [^self].
	self streamingWrite: count into: aWriteStream.
	^count! !

XTReadStream subclass: #XTSharedQueueReadStream
	instanceVariableNames: 'contentsSpecies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Xtreams-Terminals'!

!XTSharedQueueReadStream commentStamp: '<historical>' prior: 0!
Read stream on a SharedQueue. Primarily used for data transfer between processes.
{{{
	queue := SharedQueue new.
	in := queue reading.
	out := queue writing.
	received := Array new writing.
	done := Semaphore new.
	consumer :=	
		[ | size |
			[	(size := in get) isZero
			] whileFalse: [ | word |
				word := ByteString new: size.
				in read: size into: word.
				received put: word ].
			done signal.
		] fork.
	#(one two three four) do: [ :word | out put: word size; write: word ].
	out put: 0.
	done wait.
	received conclusion
}}}

Instance Variables
	contentsSpecies	<Class> species for collections of elements of this stream

!

!XTSharedQueueReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:24'!
close! !

!XTSharedQueueReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:24'!
contentsSpecies
	^contentsSpecies! !

!XTSharedQueueReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:24'!
contentsSpecies: aClass
	contentsSpecies := aClass! !

!XTSharedQueueReadStream methodsFor: 'accessing' stamp: 'nice 10/9/2010 22:24'!
get
	^source next! !

!XTSharedQueueReadStream methodsFor: 'initialize-release' stamp: 'nice 10/9/2010 22:25'!
on: aSource
	super on: aSource.
	contentsSpecies := Array! !

!XTSharedQueueReadStream methodsFor: 'accessing' stamp: 'MartinKobetic 12/30/2010 23:42'!
read: anInteger into: aSequenceableCollection at: startIndex
	startIndex to: startIndex + anInteger - 1 do: [:index | aSequenceableCollection at: index put: source next].
	^anInteger! !

!XTSharedQueueReadStream methodsFor: 'private' stamp: 'nice 10/9/2010 22:25'!
streamingInsertInto: aWriteStream
	self shouldNotImplement! !

!XTSharedQueueReadStream methodsFor: 'private' stamp: 'nice 10/9/2010 22:25'!
streamingWriteInto: aWriteStream
	self shouldNotImplement! !

!UndefinedObject methodsFor: '*Xtreams-Terminals-converting' stamp: 'nice 7/9/2011 00:48'!
writing
	^XTNullWriteStream new! !
